program genmixmem
implicit real*8 (a-h,o-z)
character outfilename*200,outseqname*200,c80tmp*80
character*200,allocatable :: infilename(:)
real*8 transvec(3),geomcen(3)
real*8 :: pi=3.141592653589793D0
integer,allocatable :: nlipid(:),irefatm(:),nlipidatm(:) !The number of each lipid type in a layer; index of reference atom and the number of atoms in each lipid type
integer,allocatable :: lipidpool(:)
integer,parameter :: nlipidatmmax=1000
real*8,allocatable :: atmxyz(:,:,:),atmxyztmp(:,:) !The first index if lipid type, then atom index, then xyz
real*8,allocatable :: randarr(:,:) !Random number array
character*4,allocatable :: atmname(:,:)
character*3,allocatable :: atmele(:,:),resname(:,:)
character chainid
logical alive

write(*,"(a)") " genmixmem: A utility to flexibly generate bilayer membrane consisted of various type of molecules"
write(*,*) "Version 1.2  Release date: 2019-Jun-15"
write(*,*) "Programmed by Tian Lu (sobereva@sina.com)"
write(*,"(a)") " Please cite as: Tian Lu, genmixmem program, http://sobereva.com/245 (accessed month day, year)"
write(*,*)
inquire(file="input.txt",exist=alive)
if (.not.alive) then
	write(*,*) "Error: Cannot find ""input.txt"" in current folder!"
    write(*,*) "Press ENTER button to exit"
    read(*,*)
    stop
end if
open(10,file="input.txt",status="old")
read(10,*) outfilename
read(10,*) ntype !Number of lipid types
read(10,*) nlipidtot !Number of lipid in each layer
read(10,*) boxlen
read(10,*) irandrot
read(10,*) refz1,refz2
read(10,*) identpat
read(10,*)
read(10,*)
read(10,*)
allocate(infilename(ntype),nlipid(ntype),irefatm(ntype),nlipidatm(ntype))
allocate(atmxyz(ntype,nlipidatmmax,3),atmxyztmp(nlipidatmmax,3),atmname(ntype,nlipidatmmax),atmele(ntype,nlipidatmmax),resname(ntype,nlipidatmmax))
do i=1,ntype
	read(10,*) infilename(i),nlipid(i),irefatm(i)
end do
close(10)

!Load lipid molecule
do itype=1,ntype
	open(10,file=infilename(itype),status="old")
	write(*,*) "Loading "//trim(infilename(itype))
	istart=0
	itmp=0
	do while(.true.)
		itmp=itmp+1
		read(10,"(a)",iostat=ierror) c80tmp
		if (istart==0.and.(index(c80tmp(1:6),"HETATM")/=0.or.index(c80tmp(1:6),"ATOM")/=0)) istart=itmp
		if ((istart/=0.and.(index(c80tmp(1:6),"HETATM")==0.and.index(c80tmp(1:6),"ATOM")==0)).or.ierror/=0) then
			iend=itmp-1
			exit
		end if
	end do
	nlipidatm(itype)=iend-istart+1
	write(*,"(' The number of atoms in this pdb file:',i6,/)") nlipidatm(itype)
	rewind(10)
	do i=1,istart-1
		read(10,*)
	end do
	iatm=0
	do icyc=istart,iend
		iatm=iatm+1
		read(10,"(12x,a4,1x,a3,10x,3f8.3,22x,a3)") atmname(itype,iatm),resname(itype,iatm),atmxyz(itype,iatm,:),atmele(itype,iatm)
	end do
	close(10)
	!Move geometry center of lipid to origin point
	n=nlipidatm(itype)
	geomcen(1)=sum(atmxyz(itype,:n,1))/nlipidatm(itype)
	geomcen(2)=sum(atmxyz(itype,:n,2))/nlipidatm(itype)
	geomcen(3)=sum(atmxyz(itype,:n,3))/nlipidatm(itype)
	do i=1,nlipidatm(itype)
		atmxyz(itype,i,:)=atmxyz(itype,i,:)-geomcen
	end do
end do


if (nlipidtot/=sum(nlipid(:))) then
	write(*,"(a,i4,a)") " Error: The sum of the number of lipids of each type",sum(nlipid(:))," is unequal to the total number of lipids you set!"
	pause
	stop
end if

nlipidside=int(dsqrt(dfloat(nlipidtot)))
write(*,"(/,' The number of lipids in each layer:',i5,/)") nlipidside
allocate(lipidpool(nlipidtot),randarr(nlipidside,nlipidside))

itmp=index(outfilename,'.')
outseqname=outfilename(:itmp)//"txt"

open(10,file=outfilename,status="replace")
open(11,file=outseqname,status="replace")
iresid=0
iatm=0
stepval=boxlen/nlipidside
do itime=1,2 !1st and 2nd layer
	if (itime==1) then
		chainid='A'
	else if (itime==2) then
		chainid='B'
		!Rotate all lipid coordinates w.r.t. X axis by 180 degree
		do itype=1,ntype
			atmxyz(itype,:,2)=-atmxyz(itype,:,2)
			atmxyz(itype,:,3)=-atmxyz(itype,:,3)
		end do
	end if
	!Generate temporary array for locating lipid position
	itmp=1
	do itype=1,ntype
		lipidpool(itmp:itmp+nlipid(itype)-1)=itype
		itmp=itmp+nlipid(itype)
	end do
	nremain=nlipidtot
	!Generate random number array
	if (itime==1.or.(itime==2.and.identpat==0)) then
		call RANDOM_SEED()
		call RANDOM_NUMBER(randarr)
	end if
	
	do i=1,nlipidside
		do j=1,nlipidside
			iresid=iresid+1
			!Randomly select a lipid to place it to (i,j) slot
			idx=int(randarr(i,j)*nremain)+1
			itype=lipidpool(idx)
			write(11,"(1x,a,i2)") resname(itype,1),1
			lipidpool(idx:nremain-1)=lipidpool(idx+1:nremain)
			nremain=nremain-1
			!Determine and translate lipid coordinate
			currx=stepval*(i-1)
			curry=stepval*(j-1)
			transvec(1)=currx-atmxyz(itype,irefatm(itype),1)
			transvec(2)=curry-atmxyz(itype,irefatm(itype),2)
			if (itime==1) transvec(3)=refz1-atmxyz(itype,irefatm(itype),3)
			if (itime==2) transvec(3)=refz2-atmxyz(itype,irefatm(itype),3)
			
			if (irandrot==1) then
				CALL RANDOM_NUMBER(rotval) !Randomly rotate molecule
				rotval=rotval*2*pi
			else if (irandrot==0) then
				rotval=0
			else if (irandrot==-1.and.itime==2) then
				rotval=pi/4
			else if (irandrot==-2.and.itime==2) then
				rotval=pi/2
			else if (irandrot==-3.and.itime==2) then
				rotval=pi
			end if
			atmxyztmp(:,1)=cos(rotval)*atmxyz(itype,:,1)-sin(rotval)*atmxyz(itype,:,2)
			atmxyztmp(:,2)=sin(rotval)*atmxyz(itype,:,1)+cos(rotval)*atmxyz(itype,:,2)
			atmxyztmp(:,3)=atmxyz(itype,:,3)
						
			do itmp=1,nlipidatm(itype)
				iatm=iatm+1
				write(10,"(a,i5,1x,a4,1x,a,1x,a,i4,4x,3f8.3,2f6.2,10x,a)") &
				"HETATM",iatm,atmname(itype,itmp),resname(itype,itmp),chainid,iresid,atmxyztmp(itmp,:)+transvec,1.0,0.0,atmele(itype,itmp)
			end do
			
		end do
	end do
	
end do

close(10)
close(10)

write(*,"(a)") " Done, the structure has been outputted to "//trim(outfilename)//" in current folder. &
The residue sequence has been outputted to "//trim(outseqname)//" in current folder."
write(*,*) "Press ENTER button to exit"
read(*,*)

end program